// Copyright 1996, Marimba Inc. All Rights Reserved.
// Please feel free to use this code as an example.

import java.awt.*;
import java.net.*;
import marimba.gui.*;

/**
 * An example of a temperature conversion applet.
 * The GUI for this applet is created with Bongo
 * and is stored in a filed called <b>temp.gui</b>.
 * @author Arthur van Hoff
 */
public class TempApplet extends java.applet.Applet {
    Presentation presentation;
    PlayerPanel player;
    PlayerUtil util;

    /**
     * Initialize the applet
     */
    public void init() {
	System.out.println("INIT");

	// Load the Presentation
	try {
	    presentation = Presentation.getPresentation(new URL(getDocumentBase(), "temp.gui"));
	} catch (MalformedURLException e) {
	    e.printStackTrace();
	}

	System.out.println("PRESENTATION: " + presentation);

	// Create a Player Panel
	setLayout(new BorderLayout());
	add("Center", player = new PlayerPanel());

	// Set the presentation
	player.setPresentation(presentation);

	// Create a Player utillity object
	util = new PlayerUtil(player);
	System.out.println("INIT DONE");
    }

    /**
     * Handle events. These events will end up here
     * from the presentation because they are send
     * to the parent.
     */
    public boolean handleEvent(Event evt) {
	if (evt.target instanceof Widget) {
	    Widget w = (Widget)evt.target;
	    String nm = w.getName();

	    switch (evt.id) {
	      case Event.SCROLL_LINE_UP:
	      case Event.SCROLL_LINE_DOWN:
	      case Event.SCROLL_PAGE_UP:
	      case Event.SCROLL_PAGE_DOWN:
	      case Event.SCROLL_ABSOLUTE:
		if ("slider-C".equals(nm)) {
		    int c = ((SliderWidget)w).getIntegerValue();
		    int f = c * 9 / 5 + 32;
		    util.setValue("value-C", new Integer(c));
		    util.setValue("value-F", new Integer(f));
		    util.setValue("slider-F", new Integer(f));
		    return true;
		}
		if ("slider-F".equals(nm)) {
		    int f = ((SliderWidget)w).getIntegerValue();
		    int c = (f - 32) * 5 / 9;
		    util.setValue("value-F", new Integer(f));
		    util.setValue("value-C", new Integer(c));
		    util.setValue("slider-C", new Integer(c));
		    return true;
		}
		break;
	    }
	}
	return super.handleEvent(evt);
    }
}
