// Copyright 1996, Marimba Inc. All Rights Reserved.
// Please feel free to use this code as an example.

import java.io.*;
import marimba.io.*;
import marimba.persist.*;

/**
 * This program illustrates the saving and loading of
 * a simple data structure to and from a file.
 *
 * @author Arthur van Hoff
 */
public class BinaryTreeNode implements PropertyObject {
    public String  value;
    public BinaryTreeNode  left;
    public BinaryTreeNode  right;


    /**
     * Constructor.
     */
    public BinaryTreeNode() {
    }

    /**
     * Constructor that set the value.
     */
    public BinaryTreeNode(String value) {
	this(value, null, null);
    }

    /**
     * Constructor that sets the value and assigns a left and right child.
     */
    public BinaryTreeNode(String value, BinaryTreeNode left, BinaryTreeNode right) {
	this.value = value;
	this.left = left;
	this.right = right;
    }

    public void getProperties(PropertyList list) {
	// store properties in the list
	list.setString("value", value, "");
	list.setObject("left", left, null);
	list.setObject("right", right, null);
    }

    public void setProperties(PropertyList list) {
	// retrieve properties from list
	value = list.getString("value", "");
	left = (BinaryTreeNode)list.getObject("left", null);
	right = (BinaryTreeNode)list.getObject("right", null);
    }

    public void print(PrintStream out, int n) {
	for (int i = 0 ; i < n ; i++) {
	    out.print("  ");
	}
	out.println(value + "/" + Math.abs(hashCode()));
	if (left != null) {
	    left.print(out, n+1);
	}
	if (right != null) {
	    right.print(out, n+1);
	}
    }

    public static void main(String argv[]) throws IOException, SyntaxError {
	// Create the tree
	BinaryTreeNode node =
	    new BinaryTreeNode("Josh");

	BinaryTreeNode tree =
	    new BinaryTreeNode("Arthur",
		new BinaryTreeNode("Jonathan",
		    new BinaryTreeNode("Maurice", node, node),
		    new BinaryTreeNode("Sami", node, null)),
		node);

	// Print the tree
	System.out.println("-- before --");
	tree.print(System.out, 0);
	System.out.println();

	// Save the tree
	System.out.println("Saving...");
	FastOutputStream out = new FastOutputStream("tree.dat");
	out.writeObject(tree);
	out.close();
	System.out.println();

	// Load the tree
	System.out.println("Loading...");
	FastInputStream in = new FastInputStream("tree.dat");
	BinaryTreeNode t = (BinaryTreeNode)in.readObject();
	in.close();
	System.out.println();

	// Print the tree
	System.out.println("-- after --");
	t.print(System.out, 0);
	System.out.println();
    }
}
