// Copyright 1996, Marimba Inc. All Rights Reserved.
// Please feel free to use this code as an example.

import java.io.*;
import java.awt.*;
import marimba.gui.*;

/**
 * This program uses the presentation "username.gui" to
 * ask the user for their name. It is a simple example of
 * how to write a main Java program which uses a previously
 * prepared presentation.
 * <p>
 * It uses the PlayerFrame class to display the presentation.
 * This class is a frame containing a PlayerPanel, and a PlayerUtil
 * instance. The util instance is used to access components in the
 * displayed presentation.
 * <p>
 * You can also use scripting to achieve many of the same goals, but
 * this is an easy way to seperate the semantics from the presentation.
 *
 * @author Arthur van Hoff
 */
public class UserName extends PlayerFrame {
    /**
     * Handle events that occur in the presentation.
     */
    public boolean handleEvent(Event evt) {
	if ((evt.id == Event.ACTION_EVENT) && (evt.target instanceof Widget)) {
	    Widget w = (Widget)evt.target;
	    String nm = w.getName();

	    // The user has clicked Ok, or the username was entered.
	    if (nm.equals("okButton")) {
		String name = util.getText("userName").trim();
		if (name.length() > 0) {
		    System.out.println("Name=" + name);
		    System.exit(1);
		}
		util.setText("message", "You have to enter your name first!");
		return true;
	    }

	    // The user has clicked Cancel, exit the application
	    if (nm.equals("cancelButton")) {
		System.out.println("Cancelled");
		System.exit(0);
		return true;
	    }
	}
	return super.handleEvent(evt);
    }

    /**
     * The main program.
     */
    public static void main(String argv[]) {
	// Create the frame
	UserName frm = new UserName();
	frm.util.setPresentation("username.gui");
	frm.show();
    }
}
