// Copyright 1996, Marimba Inc. All Rights Reserved.
// @(#)DropDownMenu.java, 1.24, 01/19/97

package marimba.gui;

import java.awt.Color;
import java.awt.Event;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.util.*;

/**
 * A simple drop down menu.
 *
 * @author	Klaas Waslander
 * @version 	1.24, 01/19/97
 */
public class DropDownMenu extends PopupMenu {
    /**
     * The widget that uses this DropDownMenu.
     * This class sets the current value of this widget
     * and invokes the w.action() method when a choice
     * is being made out of the available choices of this
     * DropDownMenu.
     * The value is being set using w.setValue(new Integer(int))
     * where "int" is the index of the choice that was made.
     */
    public Widget  w;

    /**
     * Constructor without setting the max number of visible items.
     * @param autoSize	if false, the width of the choicewidget is used.
     */
    public DropDownMenu(Widget w, String choices[], boolean autoSize) {
	this(w, choices, autoSize, 0);
    }

    /**
     * Constructor: Show a dropDownMenu that belongs to the given widget.
     * The items that should be in this menu are passed in the parameter
     * called "choices".
     * @param autoSize	if false, the width of the choicewidget is used.
     */
    public DropDownMenu(Widget w, String choices[], boolean autoSize, int maxVisible) {
	this.w = w;
	lineMode = SOLID;
	this.maxVisible = maxVisible;

	int width = 20;
	FontMetrics fm = getFontMetrics(w.font);
	for (int i = 0 ; i < choices.length ; i++) {
	    add(choices[i]);
	    int strw = fm.stringWidth(choices[i]);
	    if (strw + 20 > width) {
		width = strw + 20;
	    }
	}
	if ((width > w.width) || !autoSize) {
	    width = w.width;
	}
	layout();
	popup(w, 0, w.height, width);
    }

    /**
     * Select next item, do not loop around when
     * the last item has been reached.
     * @see #current
     */
    public void nextItem() {
	int current = indexOf(this.current);
	if (current < getItemCount() - 1) {
	    super.nextItem();
	}
    }

    /**
     * Select previous item, do not loop around when
     * the first item has been reached.
     * @see #current
     */
    public void previousItem() {
	int  current = indexOf(this.current);
	if (current > 0) {
	    super.previousItem();
	}
    }

    /**
     * Handle mouse events.
     */
    public boolean handleEvent(Event evt) {
	switch (evt.id) {
	  case Event.ACTION_EVENT: {
	    Widget  cont = container.content;
	    for (int i = 0 ; i < cont.nwidgets ; i++) {
		if (evt.target == cont.widgets[i]) {
		    w.setValue(new Integer(i));
		    w.action();
		    break;
		}
	    }
	    return true;
	  }

	  case Event.MOUSE_DRAG:
	    owner.postEvent(evt);
	    break;

	  case Event.MOUSE_UP:
	    // make sure that the button of dropdownlist is not down
	    ((DropDownTextBoxWidget)owner).setDown(false);
	    break;
	}
	return super.handleEvent(evt);
    }
}
